/*globals define*/
/*jslint nomen: true, regexp: true*/

/**
 * @class de_epages.shop.linkpicker
 * @author Copyright 2006-2013, epages GmbH, All Rights Reserved
 *
 * The linkpicker opens a dialog that lets you create text or image links to various
 * internal pages and files as well as links to external urls. It remembers the last used
 * tab through adding a data-type attribute to the created link.
 *
 * ### Example
 *     require(['jquery', 'de_epages/shop/linkpicker'], function ($, linkpicker) {
 *         linkpicker.open({
 *             use_absolute: true,
 *             use_ssl: false,
 *
 *             linkText: selectedText,
 *             linkUrl: selectedUrl || '',
 *             is_linkTargetBlank: true,
 *             linkType: $('#element').data('linkType'),
 *
 *             callback: function (title, url, blank, type) {
 *                 if (title && url) {
 *                     $('#container').html(
 *                         '<a href="' + url + '"' +
 *                             blank ? 'target="_blank"' : '') + '
 *                             data-link-type="' + type + '">' +
 *                                 title +
 *                         '</a>'
 *                     );
 *                 }
 *             }
 *         });
 *     })
 *
 * @uses jQuery
 * @uses Backbone
 * @uses ep
 * @uses util.storage
 * @uses linkpicker.linktextview
 * @uses linkpicker.pagetreeview
 * @uses linkpicker.filemanagerview
 * @uses linkpicker.additionalpagesview
 * @uses linkpicker.externalview
 * @uses linkpicker.tmpl
 * @uses dict
 *
 * @chainable
 * @return {Object} Instance of linkpicker
 */
define('de_epages/shop/linkpicker', [
    'jquery',
    'backbone',
    'ep',
    'util/storage',
    './linkpicker/linktextview',
    './linkpicker/pagetreeview',
    './linkpicker/filemanagerview',
    './linkpicker/additionalpagesview',
    './linkpicker/externalurlview',
    '$tmpl!de_epages/shop/linkpicker',
    '$dict!./dictionary',
    'jquery/ui/tabs',
    'ep/ui/dialog'

], function ($, Backbone, ep, storage, LinktextView, PagetreeView, FilemanagerView, AdditionalpagesView, ExternalView, template, dict) {
    'use strict';

    var View = Backbone.View.extend({

        tagName: 'div',
        template: template({}).dictParse(dict, true),
        callback: $.noop,
        activeTab: 0,

        /**
         * Called when instantiating new view
         * @private
         */
        initialize: function () {

            // creating subview instances
            this.linkTextView = new LinktextView();
            this.filemanagerView = new FilemanagerView();
            this.additionalPagesView = new AdditionalpagesView();
            this.externalUrlView = new ExternalView();
            this.pageTreeView = new PagetreeView();

            // listen to events the subviews are publishing and call appropriate functions
            this.listenTo(this.linkTextView, 'linktextchanged', function (_text) {
                this.is_linkTextChanged = true;
                this._setLinkText(_text);
            });
            this.listenTo(this.linkTextView, 'targetoptionchanged', this._setLinkTarget);

            this.listenTo(this.additionalPagesView, 'linkpicked', function (_text, _url) {
                this._setLinkText(_text);
                this._setLinkUrl(this._getCorrectUrl(_url));
                this.linkType = 'additional';
                this._clearViewDataExceptFor(this.linkType);
            });

            this.listenTo(this.pageTreeView, 'linkpicked', function (_text, _url) {
                this._setLinkText(_text);
                this._setLinkUrl(this._getCorrectUrl(_url));
                this.linkType = 'tree';
                this._clearViewDataExceptFor(this.linkType);
            });

            this.listenTo(this.filemanagerView, 'linkpicked', function (_text, _url, _preview) {
                this._setLinkText(_text);
                this._setLinkUrl(this._getCorrectUrl(_url));
                this.linkType = 'filemanager';
                this._clearViewDataExceptFor(this.linkType);
            });

            this.listenTo(this.externalUrlView, 'linkurlchanged', function (_url) {
                this._setLinkUrl(this._getCorrectUrl(_url));
                this.linkType = 'external';
                this._clearViewDataExceptFor(this.linkType);
            });

            // render the view for additional pages (radios) as soon as the ajax request is done (datafetched triggered)
            this.listenTo(this.additionalPagesView, 'datafetched', function () {
                this.additionalPagesView.render();
            });
        },

        /**
         * Opening the linkpicker dialog or creating one if it doesn't exist
         * Can be called right from the required linkpicker instance (see example)
         *
         * @param {Object}      _options                     Can be called with the following options
         * @param {Boolean}     _options.is_linkTargetBlank  Whether link should be opened in new window or not
         * @param {Boolean}     _options.use_absolute        Initialize linkpicker to return absolute urls
         * @param {Boolean}     _options.use_ssl             Initialize linkpicker to return https urls
         * @param {Function}    _options.callback            Called whenever the user clicks the dialog's *apply* button
         * @param {Object}      _options.dialog              Options to pass to the created dialog (see ep.ui.dialog for more information)
         * @param {String}      _options.callback.title      Returned link text
         * @param {String}      _options.callback.url        Returned link url
         * @param {String}      _options.callback.blank      Whether the link should open in a new window
         * @param {String}      _options.callback.type       Indicates the type of the returned link. Can be either *tree*, *additional*, *filemanager* or *external*
         *                                                   Is used to open the correct tab when editing a link
         * @param {String}      _options.linkText            Text that is displayed as the link's text
         * @param {String}      _options.linkType="external" Type of the link. Can either be *tree*, *additional*, *filemanager* or *external*
         * @param {String}      _options.linkUrl             Url the link refers to,
         * @since 6.16.1
         */
        open: function (_options) {

            this.use_absoluteUrl = _options.use_absolute;
            this.use_ssl = _options.use_ssl;

            $.extend(this, $.tmplItem(this.template).elements);

            var linkpicker = this,
                callback = _options.callback || $.noop,

                dialogOptions = _options.dialog || {

                    title: dict.translate('EditLink'),
                    width: 920,
                    height: 563,
                    draggable: false,
                    modal: true,
                    autoOpen: false,
                    buttons: [
                        {
                            text: dict.translate('Cancel'),
                            click: function () {
                                $(this).uiDialog('close');
                            }
                        }, {
                            text: dict.translate('Apply'),
                            click: function () {
                                $(this).uiDialog('close');
                                $(this).uiDialog('destroy');
                                storage.localStorage('linkpickerLastActiveTab', linkpicker._getActiveTabAndSetOptions(linkpicker.linkType));
                                callback(linkpicker.linkText, linkpicker.linkUrl, linkpicker.is_linkTargetBlank, linkpicker.linkType, linkpicker.pageTreeId);
                            }
                        }
                    ]
                },

                dialog = this.dialog;

            // (Re-)creating the dialog
            this.dialog.uiDialog(dialogOptions);
            // omitting template's and backbone's div and appending straight to the dialog
            this.render().$el.children().children().appendTo(this.dialog);
            this.dialog.uiDialog('open');

            //setting members and view options (in case the picker is opened with predefined values)
            if (_options.linkText) {

                this.linkTextView.setLinkText(_options.linkText);
                this.linkText = _options.linkText;

                this._setLinkUrl(_options.linkUrl);
                this._setLinkTarget(_options.is_linkTargetBlank);
                this.is_linkTextChanged = true;
                this.linkType = _options.linkType;
            } else {
                this.is_linkTextChanged = false;
                this._setLinkText('');
                this._setLinkUrl('');
            }

            // clear all other tabs
            this._clearViewDataExceptFor(this.linkType);

            // converting template node to tabs and selecting last open tab
            this.tabpanel.tabs({
                active: this._getActiveTabAndSetOptions(this.linkType)
            });
        },

        /**
         * Closing the dialog
         */
        close: function () {
            if (this.dialog.data('uiUiDialog')) {
                this.dialog.uiDialog('close');
            }
        },

        /**
         * Renders the main view
         * @return {Object}
         */
        render: function () {
            // appending main template
            this.$el.append(this.template);

            // rendering view for linktext
            // nice to know: when omitting backbone's root div, all events registered are also omitted!
            this.linktext.append(this.linkTextView.render().$el);

            this.pagetree.append(this.pageTreeView.render().el);
            this.filemanager.append(this.filemanagerView.render().el);
            // the view is rendered as soon as getAdditionalPages() has returned data
            this.additionalPagesView.getAdditionalPages();
            this.additionalpages.append(this.additionalPagesView.el);

            this.externalurl.append(this.externalUrlView.render().el);

            return this;
        },

        /*
         * Determine whether the given protocol is correct depending on the initialization options (use_ssl)
         * @param   {String} _protocol Protocol to be checked
         * @return  {String}           Correct protocol based on initialization options (ssl)
         * @private
         */
        _getCorrectProtocol: function (_protocol) {

            // correct the protocol if we have an internal link, ignore for externals
            if (_protocol && this.linkType !== 'external') {
                if (this.use_ssl && _protocol === 'http') {
                    _protocol = 'https';
                } else if (!this.use_ssl && _protocol === 'https') {
                    _protocol = 'http';
                } else {
                    // everything is fine, return the absolute url
                    return _protocol;
                }
            }

            // url is relative, no need to touch the protocol (whyever someone called for getting the correct protocol on a realtive url)
            return _protocol;

        },

        /*
         * Returns absolute or relative url depending on given url and initialization options of the linkpicker (use_absoluteUrl)
         * @param   {String} _url Url to be checked
         * @return  {String}      link url based on initialization options (absoluteUrl)
         * @private
         */
        _getCorrectUrl: function (_url) {

            var relativeUrlExpr = /^(\/|\?[\w\W]*)/i,
                absoluteUrlExpr = /^(http|https):\/\/([\w\W]*)/i,
                splitUrl = absoluteUrlExpr.exec(_url),
                sfUrl = ep.config.storeFrontUrl,
                url = _url,
                protocol;

            if (_url) {
                if (splitUrl) {
                    protocol = splitUrl[1];
                    url = splitUrl[2];
                }

                if (this.use_absoluteUrl) {
                    if (protocol) {
                        // we got an absolute url, return url with correct protocol
                        url = (this._getCorrectProtocol(protocol) + '://' + url);
                    } else if (this.linkType !== 'external' || (this.linkType === 'external' && _url.match(relativeUrlExpr))) {
                        // we got back a relative url or user entered correct relative url, convert to absolute and return url with correct protocol
                        if (_url.indexOf('/') === 0) {
                            // looks like we have a file
                            url = ep.config.protocolAndServer.replace(/^(http|https)/i, this.use_ssl ? 'https' : 'http') + _url;
                        } else {
                            url = sfUrl.replace(/^(http|https)/i, this.use_ssl ? 'https' : 'http') + _url;
                        }
                    } else {

                        // user probably forgot to specify the protocol but entered absolute url, add protocol (assume http)
                        url = 'http://' + _url;
                    }
                } else {
                    if (protocol) {
                        // we got back an absolute link, convert to relative
                        url = _url.replace(/^.+\?/, '?');
                    } else if (_url.indexOf('/') !== 0 && !_url.match(relativeUrlExpr)) {
                        // no file and no relative url, probably user input without protocol, assume http
                        url = 'http://' + _url;
                    } else {
                        url = _url;
                    }
                }
            }
            return url;
        },

        /*
         * Gets the active tab for rendering and sets options options based on link type
         * @param   {String} _linkType Type of the link that is currently edited.
         *                             Can be either *filemanager*, *additional*, *external* or *tree*
         * @return  {number}           Active index
         * @private
         */
        _getActiveTabAndSetOptions: function (_linkType) {

            switch (_linkType) {
            case 'filemanager':
                this.filemanagerView.setPreviewImage(this.linkUrl);
                return 1;
            case 'additional':
                this.additionalPagesView.setCheckedRadio(this.linkUrl);
                return 2;
            case 'external':
                this.externalUrlView.setUrl(this.linkUrl);
                return 3;
            case 'tree':
                return 0;
            default:
                // if link was created before new linkpicker implementation (url is given), return external tab to show url
                var tab;
                if (this.linkUrl) {
                    tab = 3;
                    this.externalUrlView.setUrl(this.linkUrl);
                } else {
                    // return last active tab
                    tab = storage.localStorage('linkpickerLastActiveTab') || 0;
                }

                return tab;
            }
        },

        /*
         * Clearing all view specific data like filemanager preview, selected radio options or entered text
         * except for the specified view/link type
         * @param  {String} _givenLinkType LinkType whose view data should NOT be cleared Can be either *filemanager*, *additional*, *external* or *tree*
         * @private
         */
        _clearViewDataExceptFor: function (_givenLinkType) {

            switch (_givenLinkType) {
            case 'filemanager':
                this.pageTreeView.clearLinkData();
                this.additionalPagesView.clearLinkData();
                this.externalUrlView.clearLinkData();
                break;
            case 'additional':
                this.filemanagerView.clearLinkData();
                this.pageTreeView.clearLinkData();
                this.externalUrlView.clearLinkData();
                break;
            case 'external':
                this.filemanagerView.clearLinkData();
                this.additionalPagesView.clearLinkData();
                this.pageTreeView.clearLinkData();
                break;
            default:
                //tree
                this.filemanagerView.clearLinkData();
                this.additionalPagesView.clearLinkData();
                this.externalUrlView.clearLinkData();
            }
        },

        /*
         * Sets the text of the chosen link
         * @param {String} _text Title to be set
         * @private
         */
        _setLinkText: function (_text) {

            var text = this.linkTextView.getLinkText();

            // input is considered not changed when its value is empty
            if (this.is_linkTextChanged && text !== '') {
                this.linkText = text;
            } else {
                this.linkTextView.setLinkText(_text);
                this.linkText = _text;
                this.is_linkTextChanged = false;
            }
        },

        /*
         * Sets the url and title attribute of the chosen link
         * @param {String} _url Url to be set
         * @private
         */
        _setLinkUrl: function (_url) {
            this.linkUrl = _url;
            this.linkTextView.setLinkUrl(_url);
        },

        /*
         * Sets the target of the link (open in new or same window)
         * @param   {Boolean} _is_targetBlank Whether link should be opened in new window
         * @private
         */
        _setLinkTarget: function (_is_targetBlank) {
            this.is_linkTargetBlank = _is_targetBlank;
            this.linkTextView.setLinkTarget(_is_targetBlank);
        }
    });

    return new View();
});